package com.bs.common.core.domain.entity;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.bs.common.annotation.Excel;
import com.bs.common.annotation.Excel.ColumnType;
import com.bs.common.annotation.Excel.Type;
import com.bs.common.annotation.Excels;
import com.bs.common.core.domain.BaseEntity;
import com.bs.common.xss.Xss;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;


import javax.validation.constraints.Email;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.Size;
import java.util.Date;
import java.util.List;

/**
 * 用户对象 sys_user
 *
 * @author bs
 */
@Data
public class SysUser extends BaseEntity
{
    private static final long serialVersionUID = 1L;

    /** 用户ID */
    @Excel(name = "用户序号", cellType = ColumnType.NUMERIC, prompt = "用户编号")
    @TableId(value = "user_id",type = IdType.AUTO)
    private Long userId;

    /** 部门ID */
    @Excel(name = "部门编号", type = Type.IMPORT)
    private Long deptId;

    @Excel(name = "所属代理商")
    @TableField(exist = false)
    private String deptName;

    /** 登陆账号 */
    @Excel(name = "登陆账号")
    @Xss(message = "登陆账号不能包含脚本字符")
    @NotBlank(message = "登陆账号不能为空")
    @Size(min = 0, max = 30, message = "用户账号长度不能超过30个字符")
    private String userName;

    @Excel(name = "用户类型")
    @ApiModelProperty(value = "用户类型")
    private String userType;

    @Excel(name = "审核状态")
    @ApiModelProperty(value = "审核状态")
    private String reviewStatus;

    /** 身份证号 */

    @Excel(name = "身份证号")
    @ApiModelProperty(value = "身份证号")
    private String identityCard;

    /** 城市 */

    @Excel(name = "城市")
    @ApiModelProperty(value = "城市")
    private String city;

    /** 注册时间 */

    @JsonFormat(pattern = "yyyy-MM-dd")
    @Excel(name = "注册时间", width = 30, dateFormat = "yyyy-MM-dd")
    @ApiModelProperty(value = "注册时间")
    private Date registrationTime;

    @Excel(name = "简介")
    @ApiModelProperty(value = "简介")
    private String userDesc;

    @Excel(name = "地址")
    @ApiModelProperty(value = "地址")
    private String address;

    @Excel(name = "公司")
    @ApiModelProperty(value = "公司")
    private String company;

    @Excel(name = "真实姓名")
    @ApiModelProperty(value = "真实姓名")
    private String realName;

    @Excel(name = "标签")
    @ApiModelProperty(value = "标签")
    private String label;


    /** 用户昵称 */
    @Excel(name = "用户昵称")
    @Xss(message = "用户昵称不能包含脚本字符")
    @Size(min = 0, max = 30, message = "用户昵称长度不能超过30个字符")
    private String nickName;

    /** 用户邮箱 */
    @Excel(name = "用户邮箱")
    @Email(message = "邮箱格式不正确")
    @Size(min = 0, max = 50, message = "邮箱长度不能超过50个字符")
    private String email;

    /** 手机号码 */
    @Excel(name = "手机号码")
    @Size(min = 0, max = 11, message = "手机号码长度不能超过11个字符")
    private String phonenumber;

    /** 用户性别 */
    @Excel(name = "用户性别", readConverterExp = "0=男,1=女,2=未知")
    private String sex;

    /** 用户头像 */
    private String avatar;

    /** 密码 */
    private String password;

    /** 帐号状态（0正常 1停用） */
    @Excel(name = "帐号状态", readConverterExp = "0=正常,1=停用")
    private String status;

    /** 删除标志（0代表存在 2代表删除） */
    private String delFlag;

    /** 最后登录IP */
    @Excel(name = "最后登录IP", type = Type.EXPORT)
    private String loginIp;


    /** 是否是租户，Y是N，不是 */
//    @Excel(name = "是否是租户，Y是N，不是", type = Type.EXPORT)
    private String isTenant;

    /** 是否是租户管理员，Y是N，不是 */
//    @Excel(name = "是否是租户管理员，Y是N，不是", type = Type.EXPORT)
    private String isTenantAdmin;

    /** 最后登录时间 */
    @Excel(name = "最后登录时间", width = 30, dateFormat = "yyyy-MM-dd HH:mm:ss", type = Type.EXPORT)
    private Date loginDate;

    /** 部门对象 */
    @Excels({
        @Excel(name = "部门名称", targetAttr = "deptName", type = Type.EXPORT),
        @Excel(name = "部门负责人", targetAttr = "leader", type = Type.EXPORT)
    })
    @TableField(exist = false)
    private SysDept dept;

    /** 角色对象 */
    @TableField(exist = false)
    private List<SysRole> roles;

    /** 角色组 */
    @TableField(exist = false)
    private Long[] roleIds;

    /** 岗位组 */
    @TableField(exist = false)
    private Long[] postIds;

    @TableField(exist = false)
    private Object files;

    @TableField(exist = false)
    private String userIdNew;

    /** 角色ID */
    @TableField(exist = false)
    private Long roleId;
    private String remark;
    public SysUser()
    {

    }

    public SysUser(Long userId)
    {
        this.userId = userId;
    }

    public boolean isAdmin()
    {
        return isAdmin(this.userId);
    }

    public static boolean isAdmin(Long userId)
    {
        return userId != null && 1L == userId;
    }

    @TableField(exist = false)
    private String[] deptIds;

    @TableField(exist = false)
    private Long[] userIds;

    //1为查询所有
    @TableField(exist = false)
    private String selectAll;

    //1为查询所有
    @TableField(exist = false)
    private String isBroker;

    @TableField(exist = false)
    private String hasBroker;


    /**
     * 微信小程序openId
     */
    private String xcxOpenId;
    /**
     * 微信公众号openId
     */
    private String gzhOpenId;
    /**
     * unionId 微信唯一标识
     */
    private String unionId;
    /**
     * 微信号
     */
    private String wechatNo;


}
