package com.bs.system.mapper;

import com.baomidou.mybatisplus.annotation.InterceptorIgnore;
import com.bs.common.core.domain.entity.SysUser;
import com.bs.common.mybatis.mapper.BaseMapperX;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 用户表 数据层
 *
 * @author bs
 */
@Mapper
public interface SysUserMapper extends BaseMapperX<SysUser> {

    /**
     * 根据条件分页查询用户列表
     *
     * @param sysUser 用户信息
     * @return 用户信息集合信息
     */
    public List<SysUser> selectUserList(SysUser sysUser);

    /**
     * 根据条件分页查询已配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    public List<SysUser> selectAllocatedList(SysUser user);

    /**
     * 根据条件分页查询未分配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    public List<SysUser> selectUnallocatedList(SysUser user);



    /**
     * 校验登陆账号是否唯一
     *
     * @param userName 登陆账号
     * @return 结果
     */
    public SysUser checkUserNameUnique(String userName);

    @InterceptorIgnore(tenantLine = "on")
    public SysUser selectTenantAdmin(Long tenantId);

    /**
     * 校验手机号码是否唯一
     *
     * @param phonenumber 手机号码
     * @return 结果
     */
    public SysUser checkPhoneUnique(String phonenumber);

    /**
     * 校验email是否唯一
     *
     * @param email 用户邮箱
     * @return 结果
     */
    public SysUser checkEmailUnique(String email);

    Boolean insertNew(@Param("user")SysUser user);

}
